// STLVarArrayUtils.cpp: implementation of the CSTLVarArrayUtils class.
//
// Source code routines to insert into your program to convert from
// STL vectors to Variant arrays and vice versa.  Only converts vectors 
// of type string, long, int, float or double.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "STLVarArrayUtils.h"
#include <stdio.h>
#include <comdef.h>
#include <string>
#include <vector>

#pragma warning( disable : 4786 )

using namespace std;
using namespace std::rel_ops ;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSTLVarArrayUtils::CSTLVarArrayUtils()
{

}

CSTLVarArrayUtils::~CSTLVarArrayUtils()
{

}



/**********************************************************************************/

////////////////////////////////////////////////
// Convert <long> vector into Variant Array.
// The input variant should already be
// initialized and cleared.
////////////////////////////////////////////////
void CSTLVarArrayUtils::VectorToVarArray(vector<long> &stlVec, VARIANT *pv)
{
	vector<long>::iterator iter;
	LPSAFEARRAY psa;
    SAFEARRAYBOUND rgsabound[1];
	long lCount;
	long* lArrayData;
	HRESULT hr;

	// Set Variant Type out
	V_VT(pv) = VT_ARRAY | VT_I4;
	rgsabound[0].cElements = stlVec.size();
	rgsabound[0].lLbound = 0;
	psa = SafeArrayCreate(VT_I4, 1, rgsabound);
	// Lock the safe array data 
	hr = SafeArrayAccessData(psa, (LPVOID*)&lArrayData);
	if (FAILED(hr)) {
		V_VT(pv) = VT_EMPTY;
		return;
	}
	for (lCount=0,iter=stlVec.begin(); iter!=stlVec.end(); iter++,lCount++) {
		lArrayData[lCount] = *iter;
	}
	// Unlock the safe array data 
	SafeArrayUnaccessData(psa);
	// Set the parray member of the VARIANT to the newlycreated SAFEARRAY psa
	V_ARRAY(pv) = psa;
}

////////////////////////////////////////////////
// Convert a Variant into a <string> vector.
// 
// The destination vector will be cleared.
// The variant must be of type variant or string.
// If the variant array holds variant, the contents
// will be checked to see if it holds a string.
////////////////////////////////////////////////
void CSTLVarArrayUtils::VarArrayToVector(vector<string> &stlVec, VARIANT v)
{
	LPSAFEARRAY psa;
	long lsaUbound, i;
	BSTR *p_bstr=NULL;
	string s;
	char *lpBuffer=NULL;
	HRESULT hr;

	stlVec.clear();
	if (v.vt == VT_EMPTY || v.parray == 0) return;

	psa = v.parray;
	lsaUbound = psa->rgsabound[0].cElements;
	if (v.vt == VT_ARRAY + VT_BSTR) {
		hr = SafeArrayAccessData (psa, (void**) &p_bstr);
		if (FAILED(hr)) return;
		try {
			for (i = 0; i < lsaUbound; i++) {
				stlVec.push_back ((char*)_bstr_t(p_bstr[i]));
			}
		} catch (...) {
			throw;
		}
		SafeArrayUnaccessData (psa);
	}
	if (v.vt == VT_ARRAY + VT_VARIANT) {
		VARIANT* vArrData;
		hr = SafeArrayAccessData (psa, (void**) & vArrData);
		if (FAILED(hr)) return;
		for (i = 0; i < lsaUbound; i++) {
			if (vArrData[i].bstrVal) {
				stlVec.push_back ((char*)_bstr_t(vArrData[i].bstrVal));
			}
		}
		SafeArrayUnaccessData (psa);
	}
}	

